/******************************************************************************
*
* DEFCON 17 BADGE
*
* Filename:		  DC17_Badge.c
* Author:		    Joe Grand (Kingpin)
* Revision:		  1.0
* Last Updated: April 27, 2009
*
* Description:	Main File for the DEFCON 17 Badge (Freescale MC56F8006)
* Notes:
*
* See DC17_Badge.h for more inf0z...		
*******************************************************************************/

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "PWMRed.h"
#include "PWMGreen.h"
#include "PWMBlue.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "DC17_Badge.h"


/****************************************************************************
 ************************** Global variables ********************************
 ***************************************************************************/

// Badge 
//badge_state_type badge_state = I2C; // begin in I2C (multi-badge) mode on power-up
//badge_type_struct badge_type;
struct packed_flags flags;

// FFT
dfr16_tRFFTStruct RFFT;
dfr16_tRFFTStruct *pRFFT = &RFFT;
CFrac16 FFTInplaceBuf[MAX_RFFT_LEN/2];
unsigned long FFTVal[MAX_RFFT_LEN/2];
unsigned char pwmvals[MAX_RFFT_LEN/2];
unsigned short pwmcounter;

// A/D
unsigned int AdcBuffer[ADC_MAX_BUFFER_SIZE];
unsigned int AdcWriteIndex = 0;


/****************************************************************************
 ************************** Functions ***************************************
 ***************************************************************************/
inline void set_led(int led) {
	switch(led) {
		case 0: led1_SetVal(); return;
		case 1: led2_SetVal(); return;
		case 2: led3_SetVal(); return;
		case 3: led4_SetVal(); return;
		case 4: led5_SetVal(); return;
		case 5: led6_SetVal(); return;
		case 6: led7_SetVal(); return;
		case 7: led8_SetVal(); return;
		case 8: led9_SetVal(); return;
		case 9: led10_SetVal(); return;
		case 10: led11_SetVal(); return;
		case 11: led12_SetVal(); return;
		case 12: led13_SetVal(); return;				
		case 13: led14_SetVal(); return;				
		case 14: led15_SetVal(); return;				
	}
}

inline void clear_led(int led) {
	switch(led) {
		case 0: led1_ClrVal(); return;
		case 1: led2_ClrVal(); return;
		case 2: led3_ClrVal(); return;
		case 3: led4_ClrVal(); return;
		case 4: led5_ClrVal(); return;
		case 5: led6_ClrVal(); return;
		case 6: led7_ClrVal(); return;
		case 7: led8_ClrVal(); return;
		case 8: led9_ClrVal(); return;
		case 9: led10_ClrVal(); return;
		case 10: led11_ClrVal(); return;
		case 11: led12_ClrVal(); return;
		case 12: led13_ClrVal(); return;				
		case 13: led14_ClrVal(); return;				
		case 14: led15_ClrVal(); return;				
	}
}


void dc17_badge(void)
{ 
	int i = 0;
	dc17_init();
	  
	// check to see if badges are connected to each other and, if so, enter badge-to-badge mode (master)
  // this happens only once on power-up
  // every slave badge that is connected to the bus *must* be powered on or the bus will fail like a whale
//	if (badge_type == HUMAN) dc17_i2c_check(); 
  
  // begin normal operation
//  badge_state = FFT; // enable FFT-related functions in the interrupts


	PWMRed_SetRatio16(~1);
	PWMGreen_SetRatio16(~1);
	PWMBlue_SetRatio16(~1);

	
  while(1)
  {

		dc17_fft();
//    dc17_sleep();

  }
}

/**************************************************************/




/**************************************************************/

void dc17_fft(void) // check to see if we should do the FFT (done in main loop, eg background thread while interupts handle the rest)
{
	unsigned int  i;
	unsigned long d_avg = 0;
	unsigned long avg = 0;
	unsigned long h_avg = 0;
	unsigned long newval;
	
	Result 				res;

	unsigned long   Temp1, Temp2;    

	if (flags.dofft) {
		// perform the FFT (N = 8)
		res = dfr16RFFT (pRFFT, (Frac16 *)&AdcBuffer[0], (dfr16_sInplaceCRFFT *)&FFTInplaceBuf[0]);

		//fft done, get more samples!
		flags.dofft = 0;
		flags.doadc = 1;
		flags.draw = 0;
		for (i = 1; i <= ((MAX_RFFT_LEN/2) - 1); i++) { // ignore bin 0
			// calculate the power of the ith element (bin) of the FFT
			// M = sqrt(real^2 + imag^2)
			// P = M^2
			// since the power of a frequency element of the FFT is the square of the magnitude, we can just skip
			// the square root while calculating the magnitude and we're left with the power. 
			// we ignore the phase component of the FFT.     	
			Temp1 = abs_s(FFTInplaceBuf[i].real);
			Temp1 = Temp1*Temp1;

			Temp2 = abs_s(FFTInplaceBuf[i].imag);
			
			FFTVal[i-1] = ((Temp2*Temp2) + Temp1);
		}
		

		/*Term1_SendStr("\n\r0: ");
		Term1_SendNum((long)FFTVal[0]);
		Term1_SendStr("\t1: ");
		Term1_SendNum((long)FFTVal[1]);
		Term1_SendStr("\t2: ");
		Term1_SendNum((long)FFTVal[2]);*/
		
		for (i = 0; i < MAX_RFFT_LEN/2; ++i)
		{
		  	FFTVal[i] >>= 2; // scale FFT value down to something more reasonable (we lose resolution at lower volumes, but that's OK, we're at the disco!)
			if (FFTVal[i] > 0xB000) FFTVal[i] = 0xB000; // and limit the maximum brightness
			avg += FFTVal[i];
		}
		
		//avg >>= 4; //div 16 for avg
		d_avg = avg >> 1;
		avg = d_avg >> 2;
		h_avg = avg >> 2;
		for (i = 0; i < MAX_RFFT_LEN/2; ++i) {
			if (FFTVal[i] < 5000) {
				newval = 0;
			} else if (FFTVal[i] > d_avg)
				newval = 8;
			else if (FFTVal[i] > avg)
				newval = 2;
			else if (FFTVal[i] > avg)
				newval = 1;
			else
				newval = 0;
			pwmvals[i] = (pwmvals[i]+pwmvals[i]+pwmvals[i] + newval)>>2;
		}


		pwmcounter = 0;
		flags.draw = 1;
		// set LED colors and brightness based on input frequency			
		PWMRed_SetRatio16(~(unsigned int)(FFTVal[1]+FFTVal[2])>>1); 
		PWMGreen_SetRatio16(~(unsigned int)(FFTVal[3]+FFTVal[4])>>1); 
		PWMBlue_SetRatio16(~(unsigned int)(FFTVal[5]+FFTVal[6])>>1);
	}
}


void dc17_process_fft(void) // do stuff based on the calculated FFT data
{

/*
	unsigned char i;

	// if sound energy is greater than our sleep threshold...
	if ((FFTVal[0] > SLEEP_THRESH) || (FFTVal[1] > SLEEP_THRESH) || (FFTVal[2] > SLEEP_THRESH))
	{
		threshold_failure++; // increment counter
 		flags.sleep = 0;		 // stay awake
		TI1_Enable();				 // make sure TMR1 is only enabled when the device is completely awake (to avoid flickering LED while sleeping)
	}
	
	if (!flags.sleep) // if we're awake
	{
		// super secret frequency detection code
		if ((FFTVal[0] > (FFTVal[1] << 4)) && (FFTVal[0] > (FFTVal[2] << 4))) 	// if bin 0 is an order of magnitude larger than the other bins
		{
			++freqcount;
			if (freqcount == 42)
			{
				morse((char *)sec);
				freqcount = 0;
			}
		}
		else
			freqcount = 0;
				
		// choose our operating mode	
		// if we are in a loud environment (like a disco!), enter dance mode
		if (((FFTVal[0] > DANCE_THRESH) && (FFTVal[1] > DANCE_THRESH) && (FFTVal[2] > DANCE_THRESH)) || dancetmr) // all bins must be above the threshold to enter the mode
		{
		  // hysteresis to stay in dance mode for some amount of time after we go below the threshold
		  if (dancetmr) 
		    dancetmr--;
		  else 							// dancetmr = 0, so we've just entered this mode
		  	dancetmr = DANCE_TMR;	// set our timer

			if ((FFTVal[0] > (DANCE_THRESH >> 2)) || (FFTVal[1] > (DANCE_THRESH >> 2)) || (FFTVal[2] > (DANCE_THRESH >> 2)))
		  	dancetmr = DANCE_TMR; // reset the timer if any one bin has met grester than 1/2 threshold	 
		  	    
			for (i = 0; i < 3; ++i)
			{
			  FFTVal[i] >>= 2; // scale FFT value down to something more reasonable (we lose resolution at lower volumes, but that's OK, we're at the disco!)
				if (FFTVal[i] > 0xB000) FFTVal[i] = 0xB000; // and limit the maximum brightness
			}	
		
			// set LED colors and brightness based on input frequency			
			PWMRed_SetRatio16(~(unsigned int)FFTVal[0]); 
			PWMGreen_SetRatio16(~(unsigned int)FFTVal[1]); 
			PWMBlue_SetRatio16(~(unsigned int)FFTVal[2]); 
		
		  // if we've been in this mode for some amount of time, put out a mayday call (per DT!)
		  if (soscount >= SOS_TMR)
		  {
				morse("SOS");
				morse("SOS");
				morse("SOS");
				soscount = 0;				  
		  }	  
		}
		else 	// do cool RGB blending...
		{
			PWMRed_SetRatio16(~(LEDR[LEDptr>>2]));
		  PWMGreen_SetRatio16(~(LEDG[LEDptr>>2]));
		  PWMBlue_SetRatio16(~(LEDB[LEDptr>>2]));

			dancetmr = 0;
		  soscount = 0;
		}			
  }
*/
}


void dc17_sleep(void) // sleep, if required
{
/*
  unsigned long   Temp1, Temp2;    

	if (flags.sleep) // if it's time to go to sleep or continue to sleep...
	{
    // disable peripherals
   	PWMRed_Disable();
		PWMGreen_Disable();
		PWMBlue_Disable();
		MICOUT_Disable();
		TI0_Disable();
		clrRegBit(GPIO_A_PER, PE0);
		clrRegBit(GPIO_A_PER, PE1);
    clrRegBit(GPIO_A_PER, PE2);
		TI1_Disable();
		BADGEI2C_Disable();
				
		// save original PIT timing values
		Temp1 = getReg(PIT_MOD);
		Temp2 = getReg(PIT_CTRL);
				
		// set PIT to interrupt in 1 second to check for sound
		setReg(PIT_MOD, 62500); 
		setRegBitGroup(PIT_CTRL, PRESCALER, 6); 

		Cpu_SetStopMode(); // Zzzzz...
						
		// restore original PIT timing values	    
	  setReg(PIT_MOD, Temp1);
	  setReg(PIT_CTRL, Temp2);
	    
 		threshold_count = 0;
 		threshold_failure = 0;
	 	PWMRed_Enable(); 		
		PWMGreen_Enable();
		PWMBlue_Enable();	 			
		MICOUT_Enable();
		TI0_Enable();
		BADGEI2C_Enable();				
	}

	Cpu_SetWaitMode();  // snooze in reduced-power wait mode until the next interrupt
*/
}

/**************************************************************/


/**************************************************************/

void dc17_init(void) // badge start-up/initialization
{
  unsigned int 		options = FFT_SCALE_RESULTS_BY_N; // unconditionally scale down the results by n

	Cpu_EnablePLL();
	
	Cpu_DisableInt(); // disable global interrupts 	
	PWMRed_Enable();  // while we enable all the modules
	PWMGreen_Enable();
	PWMBlue_Enable();
	MICOUT_Enable();
	
	Term1_SendStr("Welcome to the DEFCON 17 Badge.\n\n\r");
	
	MICOUT_Measure(0);//start first measurement so when our sampling timer triggers, we have something to read

  		
	TI0_Enable();
	TI1_Enable();
	Cpu_EnableInt();  // re-enable global interrupts when we're done and ready for action

	flags.doadc = 1;
	flags.dofft = 0;
	flags.draw = 1;
	
	pwmcounter = 0;
      
//from fft demo project
#if (MAX_RFFT_LEN == 8)
    dfr16RFFTInit (pRFFT, 8, options);
#elif (MAX_RFFT_LEN == 16)
    dfr16RFFTInit (pRFFT, 16, options);
#elif (MAX_RFFT_LEN == 32)
    dfr16RFFTInit (pRFFT, 32, options);
#elif (MAX_RFFT_LEN == 64)
    dfr16RFFTInit (pRFFT, 64, options);
#elif (MAX_RFFT_LEN == 128)
    dfr16RFFTInit (pRFFT, 128, options);
#elif (MAX_RFFT_LEN == 256)
    dfr16RFFTInit (pRFFT, 256, options);
#else
    #error RFFT SIZE IS INCORRECT
#endif

	// initialize the I2C system for multi-badge communication
//	dc17_i2c_init();
}



/**************************************************************
/* INTERRUPT SERVICE ROUTINES
/**************************************************************/

void dc17_pit_isr(void) // Programmable Interval Timer (PIT): every 8ms, set to operate during Stop Mode
{
}

void dc17_t0_isr(void) // TMR0: A/D sampling, 8kHz (every 0.125ms) (Fmax = highest frequency that can be analyzed = Fs / 2 = 4kHz)
{
	unsigned int temp;
	if (flags.doadc) {
		MICOUT_GetValue16(&temp);
		MICOUT_Measure(0); //timer should give enough time to measure sample
		AdcBuffer[AdcWriteIndex] = temp;

		//once we've filled the buffer, switch modes to FFT		
		if (++AdcWriteIndex >= ADC_MAX_BUFFER_SIZE) {
			AdcWriteIndex = 0;
			flags.doadc = 0;
			flags.dofft = 1;
		}
	}
}

void dc17_t1_isr(void) // TMR1: red leds software pwm (WTB 40 gpio pins of hw pwm)
{

	int i;
	
	//should we skip drawing? (fft is updating our values!)
	if (flags.draw) {
		if(++pwmcounter >= 16) {
			pwmcounter = 0;
			return;
		}
		for (i = 0; i < 13; i++) {
			if (pwmvals[i] >= pwmcounter)
				set_led(i);
			else
				clear_led(i);			
		}
	}
}

/**************************************************************
*	Function: 	delay
*	Parameters: 16-bit delay value
*	Return:		none
*
*	Simple delay loops
* These are not very accurate because they could get interrupted
**************************************************************/
 
void delay_ms(unsigned int ms)  // delay the specified number of milliseconds
{
  unsigned int i;

  for (i = 0; i < ms; ++i)
  	Cpu_Delay100US(10);
}

/****************************** END OF FILE **********************************/

